;;; $DOOMDIR/config.el -*- lexical-binding: t; -*-

(setq user-full-name "Matt Nish-Lapidus"
      user-mail-address "matt@emenel.ca")

(setq doom-theme 'doom-one)

;; (setq doom-font (font-spec :family "Liga SFMono Nerd Font" :size 15)
;;       doom-big-font (font-spec :family "Liga SFMono Nerd Font" :size 20)
;;       doom-variable-pitch-font (font-spec :family "IBM Plex Sans" :size 16)
;;       doom-symbol-font (font-spec :family "Liga SFMono Nerd Font")
;;       doom-serif-font (font-spec :family "IBM Plex Sans" :size 16 :weight 'medium))

(setq doom-font (font-spec :family "IBM Plex Mono" :size 18))

(setq fancy-splash-image (concat doom-user-dir "splash.png"))

(setq +doom-dashboard-pwd-policy nil)

;; Must be used *after* the theme is loaded
(custom-set-faces
 '(default ((t (:background "#111111"))))
 '(hl-line ((t (:background "#222222"))))
 ;; '(solair-default-face ((t (:background "#111111"))))
 '(cursor ((t (:background "#FCFCFA" :foreground "black"))))
 '(line-number ((t (:foreground "#424242"))))
 '(mu4e-highlight-face ((t (:inherit nil :background "gray26" :foreground "gray85")))))

(setq display-line-numbers-type 'relative)
(setq scroll-preserve-screen-position t)
(setq global-hl-line-modes nil)

(if (daemonp)
    (add-hook! after-make-frame-functions
              (lambda (frame)
                (with-selected-frame frame
                  (scroll-bar-mode -1))))
  (scroll-bar-mode -1))

;; encodings
(setq locale-coding-system 'utf-8)
(set-terminal-coding-system 'utf-8)
(set-keyboard-coding-system 'utf-8)
(prefer-coding-system 'utf-8)

;; org dir == obsidian vault
(setq org-directory "~/Documents/Obsidian/Primary")

(use-package! direnv
  :config
  (direnv-mode))

;; Modeline
;; - add current workspace name
; - add major mode icon
(after! doom-modeline
  (setq doom-modeline-persp-name t
        doom-modeline-major-mode-icon t)
  (setq doom-modeline-total-line-number t))

(defun +default/insert-snippet ()
  (interactive)
  (cond ((modulep! :completion vertico)
         (call-interactively #'consult-yasnippet))
        (t (call-interactively #'yas-insert-snippet))))

;; some global settings
(setq shell-file-name (executable-find "bash"))
(setq dired-vc-rename-file t) ; renaming and changing files in dired now integrates with magit

(after! vterm
  (setq-default vterm-shell (executable-find "fish"))
  (setq-default explicit-shell-file-name (executable-find "fish"))
  (setq vterm-kill-buffer-on-exit t)
  (after! meow
    (meow-vterm-enable)))

;; remove org agenda from the splash screen
(assoc-delete-all "Open org-agenda" +doom-dashboard-menu-sections)

;; reduce which-key waiting
(after! which-key
  (setq which-key-idle-delay 0.3))

;; some defaults for cursor size, scrolling, and buffer handling
(setq x-stretch-cursor t
      kill-buffer-delete-auto-save-files t
      scroll-conservatively 0)

(setq-default tab-width 2)

(after! treemacs
  (setq treemacs-follow-mode t)
  (setq treemacs-project-follow-mode t))

(after! anzu
  (global-anzu-mode))

(global-so-long-mode 1)

(after! parinfer-rust-mode
  (setq parinfer-rust-auto-download nil)
  (setq parinfer-rust-preferred-mode "smart"))

(setq dap-ui-variable-length 200)

;; smart tabs set modes
(after! smart-tabs-mode
  (smart-tabs-insinuate 'c 'javascript 'python))

;; Add consult-line searches to isearch history (both regular and regex) -- can now use meow 'n' to continue search as well as isearch
(advice-add #'consult-line :after
            (lambda (&rest _)
              (when consult--line-history
                (add-to-history
                 'regexp-search-ring
                 (car consult--line-history)
                 regexp-search-ring-max)
                (add-to-history
                 'search-ring
                 (car consult--line-history)
                 search-ring-max))))

;; bibliography file location
(after! citar
  (setq! citar-bibliography '("~/Zotero/lib.bib"))
  (setq! citar-notes-paths '("~/Documents/Obsidian/Primary/Sources"))
  (setq! citar-file-note-extensions '("md")))

(add-to-list 'auto-mode-alist '("\\.svelte\\'" . svelte-mode))
(add-to-list 'auto-mode-alist '("\\.vs\\'" . glsl-mode))
(add-to-list 'auto-mode-alist '("\\.fs\\'" . glsl-mode))

;; web-mode settings
(add-hook! web-mode (web-mode-toggle-current-element-highlight))
(after! web-mode
  (setq web-mode-markup-indent-offset 2)
  (setq web-mode-css-indent-offset 2)
  (setq web-mode-code-indent-offset 2))

(after! markdown-mode
  (setq markdown-list-indent-width 2)
  (setq markdown-enable-wiki-links t))

;; fix centaur tabs to show at the right times
(after! centaur-tabs
  (setq centaur-tabs-set-bar 'right)
  (setq centaur-tabs-cycle-scope 'tabs))

;; set project paths and ignore buffers
(after! projectile
  (setq projectile-project-search-path '(("~/Projects" . 4) ("~/source" . 2) ("~/Music" . 3)))
  (setq projectile-globally-ignored-buffers
        '("*scratch*"
          "*lsp-log*"))
  (add-to-list 'projectile-globally-ignored-directories "~/Music/_samples"))

;; projectile ignoring buffers by their major mode
(setq projectile-globally-ignored-modes
      '("erc-mode"
        "help-mode"
        "completion-list-mode"
        "Buffer-menu-mode"
        "gnus-.*-mode"
        "occur-mode"))

;; mouse support for vertico buffers
(after! vertico
  (vertico-mouse-mode))

;; modes
(use-package! kbd-mode :defer t)

(after! lsp-mode
  (setq lsp-signature-render-documentation nil)

  (lsp-register-client
   (make-lsp-client :new-connection (lsp-stdio-connection "nixd")
                    :major-modes '(nix-mode)
                    :priority 0
                    :server-id 'nixd)))

(defun lsp-booster--advice-json-parse (old-fn &rest args)
  "Try to parse bytecode instead of json."
  (or
   (when (equal (following-char) ?#)
     (let ((bytecode (read (current-buffer))))
       (when (byte-code-function-p bytecode)
         (funcall bytecode))))
   (apply old-fn args)))
(advice-add (if (progn (require 'json)
                       (fboundp 'json-parse-buffer))
                'json-parse-buffer
              'json-read)
            :around
            #'lsp-booster--advice-json-parse)

(defun lsp-booster--advice-final-command (old-fn cmd &optional test?)
  "Prepend emacs-lsp-booster command to lsp CMD."
  (let ((orig-result (funcall old-fn cmd test?)))
    (if (and (not test?)                             ;; for check lsp-server-present?
             (not (file-remote-p default-directory)) ;; see lsp-resolve-final-command, it would add extra shell wrapper
             lsp-use-plists
             (not (functionp 'json-rpc-connection))  ;; native json-rpc
             (executable-find "emacs-lsp-booster"))
        (progn
          (when-let ((command-from-exec-path (executable-find (car orig-result))))  ;; resolve command from exec-path (in case not found in $PATH)
            (setcar orig-result command-from-exec-path))
          (message "Using emacs-lsp-booster for %s!" orig-result)
          (cons "emacs-lsp-booster" orig-result))
      orig-result)))
(advice-add 'lsp-resolve-final-command :around #'lsp-booster--advice-final-command)

;; sly
(after! sly
  (setq sly-lisp-implementations
        '((sbcl ("sbcl" "--dynamic-space-size" "4Gb"))
          (sbcl-renderdoc ("renderdoccmd" "capture" "-w" "sbcl"))))
  (setq sly-default-lisp 'sbcl)
  (setq sly-complete-symbol-function 'sly-flex-completions))

;; cl-collider cl-patterns emacs helpers TODO: test this
(after! lisp-mode
  (defun cl-patterns-helpers-load ()
    (interactive)
    (sly-eval-async '(cl:namestring (asdf:system-source-directory (asdf:find-system 'cl-patterns)))
                    (lambda (path)
                      (load (concat path "res/emacs/cl-patterns-helpers") nil nil nil t)
                      (load (concat path "res/emacs/cl-patterns-skeletons") nil nil nil t)))
    ;; (define-key sly-mode-map (kbd "C-c p") 'cl-patterns-play-or-end-context-or-select-pdef)
    ;; (define-key sly-mode-map (kbd "C-c P") 'cl-patterns-play-or-stop-context-or-select-pdef)
    ;; (define-key sly-mode-map (kbd "C-c s") 'cl-patterns-stop-all)
    (define-key sly-doc-map (kbd "s") 'cl-patterns-supercollider-documentation)
    (add-hook! 'sly-connected-hook 'cl-patterns-helpers-load)))

(set-popup-rule! "*sly-description*" :side 'right :size '0.52)

(after! lisp-markup
  (add-hook! 'lisp-mode 'lisp-markup-minor-mode))
(add-to-list 'auto-mode-alist '("\\.lsx\\'" . lisp-mode))

(after! nix-mode
  (set-formatter! 'alejandra '("alejandra" "--quiet") :modes '(nix-mode))
  (setopt lsp-nix-nixd-server-path "nixd"
   lsp-nix-nixd-formatting-command [ "nixpkgs-alejandra"]))

;; add platformio to ino files
(add-to-list 'auto-mode-alist '("\\.ino\\'" . arduino-mode))
(add-to-list 'auto-mode-alist '("\\.ino\\'" . platformio-mode))

(after! gcode-mode
  (add-hook! 'gcode-mode-hook 'eldoc-mode))

(use-package! tramp
  :defer t
  :config
  ;; To “turn off” the backup feature for remote files and stop TRAMP from saving to the backup directory.
  ;; See https://www.gnu.org/software/tramp/#Auto_002dsave-File-Lock-and-Backup
  (add-to-list 'backup-directory-alist
               (cons tramp-file-name-regexp nil))
  (customize-set-variable 'tramp-backup-directory-alist backup-directory-alist)
  (setq backup-enable-predicate
        (lambda (name) nil)))

;; override aggressive key shortcuts for alt-l/r
(use-package! drag-stuff
  :defer t)

(use-package! scad-dbus
  :after scad-mode)

(use-package! eglot-booster
  :after eglot
  :config (eglot-booster-mode))

(after! eglot
  (add-to-list 'eglot-server-programs '(svelte-mode . ("svelteserver" "--stdio")))
  (add-to-list 'eglot-server-programs '(scad-mode . ("openscad-lsp" "--stdio"))))

(setq-default eglot-workspace-configuration
              '(:nixd (:formatting (:command ["alejandra"]))))


;; load additional config files
(load! "obsidian-config.el")
(load! "mu4e-config.el")

;; import all my customized keybindings based on doom evil mode, but with meow instead.
(load! "bindings.el")
(load! "meow.el")

(delete-selection-mode t)               ; delete stuff!
(setq delete-active-region t)           ; delete selected stuff!
(+global-word-wrap-mode +1)             ; default to soft word wrap

(use-package! colorful-mode
  :custom
  (colorful-use-prefix t)
  ;; (colorful-only-strings 'only-prog)
  (css-fontify-colors nil)
  :config
  (global-colorful-mode t)
  (add-to-list 'global-colorful-modes 'helpful-mode)
  (setq css-fontify-colors nil
        web-mode-enable-css-colorization nil))

(add-hook! 'post-command-hook
          (lambda ()
            "delete colorful overlay on active mark"
            (when-let* (colorful-mode
                        (beg (use-region-beginning))
                        (end (use-region-end)))
              ;; Remove full colorful overlay instead only the part where
              ;; the region is.
                  (dolist (ov (overlays-in beg end))
                    (when (overlay-get ov 'colorful--overlay)
                      (delete-overlay ov))))))

(add-hook! 'deactivate-mark-hook
          (lambda ()
            "refontify deleted mark"
            (when-let* (colorful-mode
                        (beg (region-beginning))
                        (end (region-end)))
              (font-lock-flush beg end))))

;; (defun +workspaces-associate-frame-fn (frame &optional _new-frame-p)
;;   "Create a blank, new perspective and associate it with FRAME."
;;   (when persp-mode
;;     (with-selected-frame frame
;;       (if (not (cdr-safe (persp-frame-list-without-daemon)))
;;           (+workspace-switch +workspaces-main t)
;;         (+workspace-switch (format "#%s" (+workspace--generate-id)) t))
;;       (unless (doom-real-buffer-p (current-buffer))
;;         (switch-to-buffer (doom-fallback-buffer)))
;;       (set-frame-parameter frame 'workspace (+workspace-current-name))
;;       ;; ensure every buffer has a buffer-predicate
;;       (persp-set-frame-buffer-predicate frame))
;;     (run-at-time 0.1 nil #'+workspace/display)))
